/*
    Copyright 2020. Huawei Technologies Co., Ltd. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
*/

package com.huawei.agconnectauth;

import android.content.Context;

import androidx.annotation.NonNull;

import com.huawei.agconnect.AGConnectInstance;
import com.huawei.agconnect.auth.AGConnectAuth;
import com.huawei.agconnect.auth.AGConnectAuthCredential;
import com.huawei.agconnect.auth.AGConnectUser;
import com.huawei.agconnect.auth.AGConnectUserExtra;
import com.huawei.agconnect.auth.EmailAuthProvider;
import com.huawei.agconnect.auth.EmailUser;
import com.huawei.agconnect.auth.PhoneAuthProvider;
import com.huawei.agconnect.auth.PhoneUser;
import com.huawei.agconnect.auth.ProfileRequest;
import com.huawei.agconnect.auth.SelfBuildProvider;
import com.huawei.agconnect.auth.SignInResult;
import com.huawei.agconnect.auth.TokenResult;
import com.huawei.agconnect.auth.VerifyCodeResult;
import com.huawei.agconnect.auth.VerifyCodeSettings;
import com.huawei.agconnect.core.service.auth.OnTokenListener;
import com.huawei.agconnect.core.service.auth.TokenSnapshot;
import com.huawei.agconnect.exception.AGCException;
import com.huawei.hmf.tasks.OnFailureListener;
import com.huawei.hmf.tasks.OnSuccessListener;

import java.util.HashMap;
import java.util.Locale;
import java.util.Map;

import io.flutter.embedding.engine.plugins.FlutterPlugin;
import io.flutter.plugin.common.BinaryMessenger;
import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.MethodChannel.MethodCallHandler;
import io.flutter.plugin.common.MethodChannel.Result;
import io.flutter.plugin.common.PluginRegistry.Registrar;

import static com.huawei.agconnect.auth.AGCAuthException.*;

public class AGConnectAuthPlugin implements FlutterPlugin, MethodCallHandler, EventChannel.StreamHandler {
    private MethodChannel channel;
    private EventChannel eventChannel;
    private EventChannel.EventSink eventSink;

    public static void registerWith(Registrar registrar) {
        AGConnectAuthPlugin instance = new AGConnectAuthPlugin();
        instance.initPlugin(registrar.messenger(), registrar.context().getApplicationContext());
    }

    @Override
    public void onAttachedToEngine(@NonNull FlutterPluginBinding flutterPluginBinding) {
        initPlugin(flutterPluginBinding.getBinaryMessenger(), flutterPluginBinding.getApplicationContext());
    }

    @Override
    public void onDetachedFromEngine(@NonNull FlutterPluginBinding binding) {
        channel.setMethodCallHandler(null);
        eventChannel.setStreamHandler(null);
    }

    private void initPlugin(BinaryMessenger messenger, Context context) {
        if (AGConnectInstance.getInstance() == null) {
            AGConnectInstance.initialize(context);
        }

        channel = new MethodChannel(messenger, "com.huawei.flutter/agconnect_auth");
        channel.setMethodCallHandler(this);
        eventChannel = new EventChannel(messenger, "com.huawei.flutter.event/agconnect_auth");
        eventChannel.setStreamHandler(this);

        AGConnectAuth.getInstance().addTokenListener(new OnTokenListener() {
            @Override
            public void onChanged(TokenSnapshot tokenSnapshot) {
                if (eventSink != null) {
                    eventSink.success(mapFromTokenSnapshot(tokenSnapshot));
                }
            }
        });
    }

    private void handleSignIn(MethodCall call, final Result result) {
        AGConnectAuthCredential credential = getCredentialFromArguments((Map) call.arguments);
        AGConnectAuth.getInstance().signIn(credential).addOnSuccessListener(new OnSuccessListener<SignInResult>() {
            @Override
            public void onSuccess(SignInResult signInResult) {
                result.success(mapFromUser(signInResult.getUser()));
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleSignInAnonymously(MethodCall call, final Result result) {
        AGConnectAuth.getInstance().signInAnonymously().addOnSuccessListener(new OnSuccessListener<SignInResult>() {
            @Override
            public void onSuccess(SignInResult signInResult) {
                result.success(mapFromUser(signInResult.getUser()));
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleSignOut(MethodCall call, final Result result) {
        AGConnectAuth.getInstance().signOut();
        result.success(null);
    }

    private void handleDeleteUser(MethodCall call, final Result result) {
        AGConnectAuth.getInstance().deleteUser();
        result.success(null);
    }

    private void handleGetCurrentUser(MethodCall call, final Result result) {
        AGConnectUser user = AGConnectAuth.getInstance().getCurrentUser();
        result.success(mapFromUser(user));
    }

    private void handleCreateUserWithEmail(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        String email = (String) map.get("email");
        String password = (String) map.get("password");
        String verifyCode = (String) map.get("verifyCode");
        EmailUser emailUser = new EmailUser.Builder()
                .setEmail(email)
                .setPassword(password)
                .setVerifyCode(verifyCode)
                .build();
        AGConnectAuth.getInstance().createUser(emailUser).addOnSuccessListener(new OnSuccessListener<SignInResult>() {
            @Override
            public void onSuccess(SignInResult signInResult) {
                result.success(mapFromUser(signInResult.getUser()));
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleCreateUserWithPhone(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        String countryCode = (String) map.get("countryCode");
        String phoneNumber = (String) map.get("phoneNumber");
        String password = (String) map.get("password");
        String verifyCode = (String) map.get("verifyCode");
        PhoneUser phoneUser = new PhoneUser.Builder()
                .setCountryCode(countryCode)
                .setPhoneNumber(phoneNumber)
                .setPassword(password)
                .setVerifyCode(verifyCode)
                .build();
        AGConnectAuth.getInstance().createUser(phoneUser).addOnSuccessListener(new OnSuccessListener<SignInResult>() {
            @Override
            public void onSuccess(SignInResult signInResult) {
                result.success(mapFromUser(signInResult.getUser()));
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleResetPasswordWithEmail(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        String email = (String) map.get("email");
        String password = (String) map.get("password");
        String verifyCode = (String) map.get("verifyCode");
        AGConnectAuth.getInstance().resetPassword(email, password, verifyCode)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
            @Override
            public void onSuccess(Void aVoid) {
                result.success(null);
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleResetPasswordWithPhone(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        String countryCode = (String) map.get("countryCode");
        String phoneNumber = (String) map.get("phoneNumber");
        String password = (String) map.get("password");
        String verifyCode = (String) map.get("verifyCode");
        AGConnectAuth.getInstance().resetPassword(countryCode, phoneNumber, password, verifyCode)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
            @Override
            public void onSuccess(Void aVoid) {
                result.success(null);
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleLink(MethodCall call, final Result result) {
        AGConnectAuthCredential credential = getCredentialFromArguments((Map) call.arguments);
        AGConnectAuth.getInstance().getCurrentUser().link(credential)
                .addOnSuccessListener(new OnSuccessListener<SignInResult>() {
            @Override
            public void onSuccess(SignInResult signInResult) {
                result.success(mapFromUser(signInResult.getUser()));
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleUnlink(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        int provider = (int) map.get("provider");
        AGConnectAuth.getInstance().getCurrentUser().unlink(provider)
                .addOnSuccessListener(new OnSuccessListener<SignInResult>() {
            @Override
            public void onSuccess(SignInResult signInResult) {
                result.success(mapFromUser(signInResult.getUser()));
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleUpdateEmail(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        String email = (String) map.get("email");
        String verifyCode = (String) map.get("verifyCode");
        Locale locale = getLocaleFromArguments((Map) call.arguments);
        AGConnectAuth.getInstance().getCurrentUser().updateEmail(email, verifyCode, locale)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
            @Override
            public void onSuccess(Void aVoid) {
                result.success(mapFromUser(AGConnectAuth.getInstance().getCurrentUser()));
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleUpdatePhone(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        String countryCode = (String) map.get("countryCode");
        String phoneNumber = (String) map.get("phoneNumber");
        String verifyCode = (String) map.get("verifyCode");
        Locale locale = getLocaleFromArguments((Map) call.arguments);
        AGConnectAuth.getInstance().getCurrentUser().updatePhone(countryCode, phoneNumber, verifyCode, locale)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
            @Override
            public void onSuccess(Void aVoid) {
                result.success(mapFromUser(AGConnectAuth.getInstance().getCurrentUser()));
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleUpdateProfile(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        String displayName = (String) map.get("displayName");
        String photoUrl = (String) map.get("photoUrl");
        ProfileRequest request = new ProfileRequest.Builder()
                .setDisplayName(displayName)
                .setPhotoUrl(photoUrl)
                .build();
        AGConnectAuth.getInstance().getCurrentUser().updateProfile(request)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
            @Override
            public void onSuccess(Void aVoid) {
                result.success(mapFromUser(AGConnectAuth.getInstance().getCurrentUser()));
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleUpdatePassword(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        String password = (String) map.get("password");
        String verifyCode = (String) map.get("verifyCode");
        int provider = (int) map.get("provider");
        AGConnectAuth.getInstance().getCurrentUser().updatePassword(password, verifyCode, provider)
                .addOnSuccessListener(new OnSuccessListener<Void>() {
            @Override
            public void onSuccess(Void aVoid) {
                result.success(mapFromUser(AGConnectAuth.getInstance().getCurrentUser()));
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleGetUserExtra(MethodCall call, final Result result) {
        AGConnectAuth.getInstance().getCurrentUser().getUserExtra()
                .addOnSuccessListener(new OnSuccessListener<AGConnectUserExtra>() {
            @Override
            public void onSuccess(AGConnectUserExtra agConnectUserExtra) {
                Map<String, String> userExtra = new HashMap<>();
                userExtra.put("createTime", agConnectUserExtra.getCreateTime());
                userExtra.put("lastSignInTime", agConnectUserExtra.getLastSignInTime());
                Map user = mapFromUser(AGConnectAuth.getInstance().getCurrentUser());
                Map res = new HashMap<>();
                res.put("userExtra", userExtra);
                res.put("user", user);
                result.success(res);
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleGetToken(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        boolean forceRefresh = (boolean) map.get("forceRefresh");
        AGConnectAuth.getInstance().getCurrentUser().getToken(forceRefresh)
                .addOnSuccessListener(new OnSuccessListener<TokenResult>() {
            @Override
            public void onSuccess(TokenResult tokenResult) {
                Map<String, Object> res = new HashMap<>();
                res.put("token", tokenResult.getToken());
                res.put("expirePeriod", tokenResult.getExpirePeriod());
                result.success(res);
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleRequestEmailVerifyCode(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        String email = (String) map.get("email");
        VerifyCodeSettings settings = getSettingsFromArguments((Map) call.arguments);
        EmailAuthProvider.requestVerifyCode(email, settings)
                .addOnSuccessListener(new OnSuccessListener<VerifyCodeResult>() {
            @Override
            public void onSuccess(VerifyCodeResult verifyCodeResult) {
                Map<String, String> res = new HashMap<>();
                res.put("shortestInterval", verifyCodeResult.getShortestInterval());
                res.put("validityPeriod", verifyCodeResult.getValidityPeriod());
                result.success(res);
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    private void handleRequestPhoneVerifyCode(MethodCall call, final Result result) {
        Map map = (Map) call.arguments;
        String countryCode = (String) map.get("countryCode");
        String phoneNumber = (String) map.get("phoneNumber");
        VerifyCodeSettings settings = getSettingsFromArguments((Map) call.arguments);
        PhoneAuthProvider.requestVerifyCode(countryCode, phoneNumber, settings)
                .addOnSuccessListener(new OnSuccessListener<VerifyCodeResult>() {
            @Override
            public void onSuccess(VerifyCodeResult verifyCodeResult) {
                Map<String, String> res = new HashMap<>();
                res.put("shortestInterval", verifyCodeResult.getShortestInterval());
                res.put("validityPeriod", verifyCodeResult.getValidityPeriod());
                result.success(res);
            }
        }).addOnFailureListener(authFailureListener(result));
    }

    @Override
    public void onMethodCall(@NonNull MethodCall call, @NonNull Result result) {
        switch (call.method) {
            case "signIn":
                handleSignIn(call, result);
                break;
            case "signInAnonymously":
                handleSignInAnonymously(call, result);
                break;
            case "signOut":
                handleSignOut(call, result);
                break;
            case "deleteUser":
                handleDeleteUser(call, result);
                break;
            case "getCurrentUser":
                handleGetCurrentUser(call, result);
                break;
            case "createUserWithEmail":
                handleCreateUserWithEmail(call, result);
                break;
            case "createUserWithPhone":
                handleCreateUserWithPhone(call, result);
                break;
            case "resetPasswordWithEmail":
                handleResetPasswordWithEmail(call, result);
                break;
            case "resetPasswordWithPhone":
                handleResetPasswordWithPhone(call, result);
                break;
            case "link":
                handleLink(call, result);
                break;
            case "unlink":
                handleUnlink(call, result);
                break;
            case "updateEmail":
                handleUpdateEmail(call, result);
                break;
            case "updatePhone":
                handleUpdatePhone(call, result);
                break;
            case "updateProfile":
                handleUpdateProfile(call, result);
                break;
            case "updatePassword":
                handleUpdatePassword(call, result);
                break;
            case "getUserExtra":
                handleGetUserExtra(call, result);
                break;
            case "getToken":
                handleGetToken(call, result);
                break;
            case "requestEmailVerifyCode":
                handleRequestEmailVerifyCode(call, result);
                break;
            case "requestPhoneVerifyCode":
                handleRequestPhoneVerifyCode(call, result);
                break;
            default:
                result.notImplemented();
                break;
        }
    }

    @Override
    public void onListen(Object arguments, EventChannel.EventSink events) {
        eventSink = events;
    }

    @Override
    public void onCancel(Object arguments) {
        eventSink = null;
    }

    private Map<String, Object> mapFromTokenSnapshot(TokenSnapshot token) {
        if (token != null) {
            Map<String, Object> res = new HashMap<>();
            res.put("state", token.getState().ordinal());
            res.put("token", token.getToken());
            return res;
        } else {
            return null;
        }
    }

    private Map<String, Object> mapFromUser(AGConnectUser user) {
        if (user != null) {
            Map<String, Object> res = new HashMap<>();
            res.put("isAnonymous", user.isAnonymous());
            res.put("uid", user.getUid());
            res.put("email", user.getEmail());
            res.put("phone", user.getPhone());
            res.put("displayName", user.getDisplayName());
            res.put("photoUrl", user.getPhotoUrl());
            res.put("providerId", Integer.valueOf(user.getProviderId()));
            res.put("providerInfo", user.getProviderInfo());
            res.put("emailVerified", user.getEmailVerified());
            res.put("passwordSet", user.getPasswordSetted());
            return res;
        } else {
            return null;
        }
    }

    private VerifyCodeSettings getSettingsFromArguments(Map arguments) {
        Map map = (Map) arguments.get("settings");
        if (map != null) {
            int action = (int) map.get("action") == 0 ? VerifyCodeSettings.ACTION_REGISTER_LOGIN
                    : VerifyCodeSettings.ACTION_RESET_PASSWORD;
            Locale locale = getLocaleFromArguments(map);
            int interval = (int) map.get("sendInterval");
            return new VerifyCodeSettings.Builder()
                    .action(action)
                    .locale(locale)
                    .sendInterval(interval)
                    .build();
        }
        return null;
    }

    private Locale getLocaleFromArguments(Map arguments) {
        Locale locale = null;
        if (arguments != null) {
            String localeLanguage = (String) arguments.get("localeLanguage");
            String localeCountry = (String) arguments.get("localeCountry");
            if (localeLanguage != null && localeCountry != null) {
                locale = new Locale(localeLanguage, localeCountry);
            }
        }
        return locale;
    }

    private AGConnectAuthCredential getCredentialFromArguments(Map arguments) {
        Map credential = (Map) arguments.get("credential");
        if (credential == null)
            return null;
        int provider = (int) credential.get("provider");
        switch (provider) {
            case AGConnectAuthCredential.SelfBuild_Provider:
                String selfBuildToken = (String) credential.get("token");
                boolean autoCreateUser = (boolean) credential.get("autoCreateUser");
                return SelfBuildProvider.credentialWithToken(selfBuildToken, autoCreateUser);

            case AGConnectAuthCredential.Email_Provider:
                String email = (String) credential.get("email");
                String password = (String) credential.get("password");
                String verifyCode = (String) credential.get("verifyCode");
                return EmailAuthProvider.credentialWithVerifyCode(email, password, verifyCode);

            case AGConnectAuthCredential.Phone_Provider:
                String countryCode = (String) credential.get("countryCode");
                String phoneNumber = (String) credential.get("phoneNumber");
                String phonePassword = (String) credential.get("password");
                String phoneVerifyCode = (String) credential.get("verifyCode");
                return PhoneAuthProvider
                        .credentialWithVerifyCode(countryCode, phoneNumber, phonePassword, phoneVerifyCode);

            default:
                return null;
        }
    }

    private OnFailureListener authFailureListener(final Result result) {
        return new OnFailureListener() {
            @Override
            public void onFailure(Exception e) {
                Map<String, Object> map = new HashMap<>();
                if (e instanceof AGCException) {
                    int code = ((AGCException) e).getCode();
                    map.put("exceptionCode", getAuthExceptionCode(code));
                    result.error(String.valueOf(code),
                            ((AGCException) e).getErrMsg(),
                            map);
                } else {
                    result.error("", e.getMessage(), map);
                }
            }
        };
    }

    private int getAuthExceptionCode(int code) {
        switch (code) {
            case NULL_TOKEN:
                return 0;
            case NOT_SIGN_IN:
                return 1;
            case USER_LINKED:
                return 2;
            case USER_UNLINKED:
                return 3;
            case ALREADY_SIGN_IN_USER:
                return 4;
            case EMAIL_VERIFICATION_IS_EMPTY:
                return 5;
            case PHONE_VERIFICATION_IS_EMPTY:
                return 6;
            case INVALID_EMAIL:
                return 7;
            case INVALID_PHONE:
                return 8;
            case GET_UID_ERROR:
                return 9;
            case UID_PRODUCTID_NOT_MATCH:
                return 10;
            case GET_USER_INFO_ERROR:
                return 11;
            case AUTH_METHOD_NOT_SUPPORT:
                return 12;
            case PRODUCT_STATUS_ERROR:
                return 13;
            case PASSWORD_VERIFICATION_CODE_OVER_LIMIT:
                return 14;
            case INVALID_TOKEN:
                return 15;
            case INVALID_ACCESS_TOKEN:
                return 16;
            case INVALID_REFRESH_TOKEN:
                return 17;
            case TOKEN_AND_PRODUCTID_NOT_MATCH:
                return 18;
            case AUTH_METHOD_IS_DISABLED:
                return 19;
            case FAIL_TO_GET_THIRD_USER_INFO:
                return 20;
            case FAIL_TO_GET_THIRD_USER_UNION_ID:
                return 21;
            case ACCESS_TOKEN_OVER_LIMIT:
                return 22;
            case FAIL_TO_USER_LINK:
                return 23;
            case FAIL_TO_USER_UNLINK:
                return 24;
            case ANONYMOUS_SIGNIN_OVER_LIMIT:
                return 25;
            case INVALID_APPID:
                return 26;
            case INVALID_APPSECRET:
                return 27;
            case GET_QQ_USERINFO_ERROR:
                return 28;
            case QQINFO_RESPONSE_IS_NULL:
                return 29;
            case GET_QQ_UID_ERROR:
                return 30;
            case PASSWORD_VERIFY_CODE_ERROR:
                return 31;
            case GOOGLE_RESPONSE_NOT_EQUAL_APPID:
                return 32;
            case SIGNIN_USER_STATUS_ERROR:
                return 33;
            case SIGNIN_USER_PASSWORD_ERROR:
                return 34;
            case PROVIDER_USER_HAVE_BEEN_LINKED:
                return 35;
            case PROVIDER_HAVE_LINKED_ONE_USER:
                return 36;
            case FAIL_GET_PROVIDER_USER:
                return 37;
            case CANNOT_UNLINK_ONE_PROVIDER_USER:
                return 38;
            case VERIFY_CODE_INTERVAL_LIMIT:
                return 39;
            case VERIFY_CODE_EMPTY:
                return 40;
            case VERIFY_CODE_LANGUAGE_EMPTY:
                return 41;
            case VERIFY_CODE_RECEIVER_EMPTY:
                return 42;
            case VERIFY_CODE_ACTION_ERROR:
                return 43;
            case VERIFY_CODE_TIME_LIMIT:
                return 44;
            case ACCOUNT_PASSWORD_SAME:
                return 45;
            case PASSWORD_STRENGTH_LOW:
                return 46;
            case UPDATE_PASSWORD_ERROR:
                return 47;
            case PASSWORD_SAME_AS_BEFORE:
                return 48;
            case PASSWORD_IS_EMPTY:
                return 49;
            case PASSWORD_TOO_LONG:
                return 50;
            case SENSITIVE_OPERATION_TIMEOUT:
                return 51;
            case ACCOUNT_HAVE_BEEN_REGISTERED:
                return 52;
            case UPDATE_ACCOUNT_ERROR:
                return 53;
            case USER_NOT_REGISTERED:
                return 54;
            case VERIFY_CODE_ERROR:
                return 55;
            case USER_HAVE_BEEN_REGISTERED:
                return 56;
            case REGISTER_ACCOUNT_IS_EMPTY:
                return 57;
            case VERIFY_CODE_FORMAT_ERROR:
                return 58;
            case VERIFY_CODE_AND_PASSWORD_BOTH_NULL:
                return 59;
            case SEND_EMAIL_FAIL:
                return 60;
            case SEND_MESSAGE_FAIL:
                return 61;
            case CONFIG_LOCK_TIME_ERROR:
                return 62;

            default:
                return code;
        }
    }

}
