/*
    Copyright 2020. Huawei Technologies Co., Ltd. All rights reserved.
    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
*/

import 'dart:ui';

import 'package:agconnect_auth/agconnect_auth.dart';
import 'package:agconnect_auth/src/token_result.dart';

import 'platform_auth.dart';

class AGCUser {

  bool _isAnonymous;
  String _uid;
  String _email;
  String _phone;
  String _displayName;
  String _photoUrl;
  AuthProviderType _providerId;
  List<Map<String, String>> _providerInfo;
  bool _emailVerified;
  bool _passwordSet;

  AGCUser.fromMap(Map map) {
    assert(map != null);
    _updateUser(map);
  }

  void _updateUser(Map map) {
    map ??= new Map();
    this._isAnonymous = map['isAnonymous'];
    this._uid = map['uid'];
    this._email = map['email'];
    this._phone = map['phone'];
    this._displayName = map['displayName'];
    this._photoUrl = map['photoUrl'];
    this._providerId = AuthProviderType.values[map['providerId']];
    this._providerInfo = (map['providerInfo'] as List)
        ?.map((e) => (e as Map)
            .map((key, value) => MapEntry(key.toString(), value.toString())))
        ?.toList(growable: false);
    this._emailVerified = map['emailVerified'] == 1;
    this._passwordSet = map['passwordSet'] == 1;
  }

  bool get isAnonymous {
    return _isAnonymous;
  }

  String get uid {
    return _uid;
  }

  String get email {
    return _email;
  }

  String get phone {
    return _phone;
  }

  String get displayName {
    return _displayName;
  }

  String get photoUrl {
    return _photoUrl;
  }

  AuthProviderType get providerId {
    return _providerId;
  }

  List<Map<String, String>> get providerInfo {
    return _providerInfo;
  }

  bool get emailVerified {
    return _emailVerified;
  }

  bool get passwordSet {
    return _passwordSet;
  }

  Future<SignInResult> link(AGCAuthCredential credential) {
    assert(credential != null);
    return PlatformAuth.methodChannel.invokeMethod('link',
        <String, dynamic>{'credential': credential.toMap()}).then((value) {
      _updateUser(value);
      return SignInResult.fromMap(value);
    }).catchError(PlatformAuth.handlePlatformException);
  }

  Future<SignInResult> unlink(AuthProviderType provider) {
    assert(provider != null);
    return PlatformAuth.methodChannel.invokeMethod(
        'unlink', <String, dynamic>{'provider': provider.index}).then((value) {
      _updateUser(value);
      return SignInResult.fromMap(value);
    }).catchError(PlatformAuth.handlePlatformException);
  }

  Future<void> updateProfile(ProfileRequest profile) {
    assert(profile != null);
    return PlatformAuth.methodChannel.invokeMethod('updateProfile', <String, dynamic>{
      'displayName': profile.displayName,
      'photoUrl': profile.photoUrl
    }).then((value) {
      _updateUser(value);
    }).catchError(PlatformAuth.handlePlatformException);
  }

  Future<void> updateEmail(String newEmail, String newVerifyCode,
      {Locale locale}) {
    assert(newEmail != null);
    assert(newVerifyCode != null);
    return PlatformAuth.methodChannel.invokeMethod('updateEmail', <String, dynamic>{
      'email': newEmail,
      'verifyCode': newVerifyCode,
      'localeLanguage': locale?.languageCode,
      'localeCountry': locale?.countryCode,
    }).then((value) {
      _updateUser(value);
    }).catchError(PlatformAuth.handlePlatformException);
  }

  Future<void> updatePhone(
      String countryCode, String phoneNumber, String newVerifyCode,
      {Locale locale}) {
    assert(countryCode != null);
    assert(phoneNumber != null);
    assert(newVerifyCode != null);
    return PlatformAuth.methodChannel.invokeMethod('updatePhone', <String, dynamic>{
      'countryCode': countryCode,
      'phoneNumber': phoneNumber,
      'verifyCode': newVerifyCode,
      'localeLanguage': locale?.languageCode,
      'localeCountry': locale?.countryCode,
    }).then((value) {
      _updateUser(value);
    }).catchError(PlatformAuth.handlePlatformException);
  }

  Future<void> updatePassword(
      String newPassword, String verifyCode, AuthProviderType provider) {
    assert(newPassword != null);
    assert(verifyCode != null);
    assert(provider != null);
    return PlatformAuth.methodChannel.invokeMethod('updatePassword', <String, dynamic>{
      'password': newPassword,
      'verifyCode': verifyCode,
      'provider': provider.index
    }).then((value) {
      _updateUser(value);
    }).catchError(PlatformAuth.handlePlatformException);
  }

  Future<AGCUserExtra> get userExtra {
    return PlatformAuth.methodChannel.invokeMethod('getUserExtra').then((value) {
      _updateUser(value['user']);
      return AGCUserExtra.fromMap(value['userExtra']);
    }).catchError(PlatformAuth.handlePlatformException);
  }

  Future<TokenResult> getToken({bool forceRefresh = false}) {
    return PlatformAuth.methodChannel.invokeMethod('getToken', <String, dynamic>{
      'forceRefresh': forceRefresh
    }).then((value) {
      return TokenResult.fromMap(value);
    }).catchError(PlatformAuth.handlePlatformException);
  }

}
