import 'package:flutter/cupertino.dart';
import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:agconnect_auth/agconnect_auth.dart';

void main() {
  TestWidgetsFlutterBinding.ensureInitialized();

  group('userTest', () {
    const MethodChannel channel = MethodChannel('com.huawei.flutter/agconnect_auth');
    MethodCall mockCall;
    setUp(() {
      channel.setMockMethodCallHandler((MethodCall methodCall) async {
        mockCall = methodCall;
        switch (methodCall.method) {
          case 'link':
          case 'unlink':
          case 'updatePhone':
          case 'updateProfile':
          case 'updateEmail':
          case 'updatePassword':
            return {
              'isAnonymous':false,
              'uid':'test_uid',
              'email':'test_email2',
              'phone':'test_phone2',
              'displayName':'test_name2',
              'photoUrl':'test_photo2',
              'providerId':11,
              'providerInfo':[{'k2':'v2'}],
              'emailVerified':1,
              'passwordSet':1,
            };
          case 'getUserExtra':
            return {
              'user':{
            'isAnonymous':false,
            'uid':'test_uid',
            'email':'test_email2',
            'phone':'test_phone2',
            'displayName':'test_name2',
            'photoUrl':'test_photo2',
            'providerId':11,
            'providerInfo':[{'k2':'v2'}],
            'emailVerified':1,
            'passwordSet':1,
            },
              'userExtra':{
                'createTime':'test_time',
                'lastSignInTime':'test_signInTime'
              },
            };
          case 'getToken':
            return {'token':'test_token', 'expirePeriod':1234};
        }
        throw PlatformException(code: '0', message: 'Unknown method call, please update test.');
      });
      mockCall = null;
    });

    tearDown(() {
      channel.setMockMethodCallHandler(null);
    });

    Map map = {
      'isAnonymous':false,
      'uid':'test_uid',
      'email':'test_email',
      'phone':'test_phone',
      'displayName':'test_name',
      'photoUrl':'test_photo',
      'providerId':11,
      'providerInfo':[{'k1':'v1'}],
      'emailVerified':1,
      'passwordSet':1,
    };
    AGCUser user = AGCUser.fromMap(map);

    test('mapFrom', () async {
      expect(user.isAnonymous, false);
      expect(user.uid, 'test_uid');
      expect(user.email, 'test_email');
      expect(user.phone, 'test_phone');
      expect(user.displayName, 'test_name');
      expect(user.photoUrl, 'test_photo');
      expect(user.providerId, AuthProviderType.phone);
      expect(user.providerInfo, [{'k1':'v1'}]);
      expect(user.emailVerified, true);
      expect(user.passwordSet, true);
    });

    test('link', () async {
      AGCAuthCredential credential = EmailAuthProvider.credentialWithPassword('test_email2', 'test_password');
      SignInResult res = await user.link(credential);
      expect(mockCall.arguments['credential'], {'provider':12, 'email':'test_email2', 'password':'test_password', 'verifyCode':null});
      expect(res.user.uid, 'test_uid');
      expect(res.user.email, 'test_email2');
      expect(user.email, 'test_email2');
    });

    test('unlink', () async {
      SignInResult res = await user.unlink(AuthProviderType.email);
      expect(mockCall.arguments, {'provider':12});
      expect(res.user.uid, 'test_uid');
      expect(res.user.email, 'test_email2');
      expect(user.email, 'test_email2');
    });

    test('updateProfile', () async {
      ProfileRequest request = ProfileRequest('test_name2', 'test_photoUrl2');
      await user.updateProfile(request);
      expect(mockCall.arguments, {'displayName':'test_name2', 'photoUrl':'test_photoUrl2'});
      expect(user.uid, 'test_uid');
      expect(user.displayName, 'test_name2');
    });


    test('updatePhone', () async {
      await user.updatePhone('test_countryCode', 'test_phone', 'test_verifyCode', locale: Locale('zh','CN'));
      expect(mockCall.arguments, {'countryCode':'test_countryCode', 'phoneNumber':'test_phone', 'verifyCode':'test_verifyCode', 'localeLanguage':'zh', 'localeCountry':'CN'});
      expect(user.uid, 'test_uid');
      expect(user.phone, 'test_phone2');
    });

    test('updateEmail', () async {
      await user.updateEmail('test_email', 'test_verifyCode', locale: Locale('zh','CN'));
      expect(mockCall.arguments, {'email':'test_email', 'verifyCode':'test_verifyCode', 'localeLanguage':'zh', 'localeCountry':'CN'});
      expect(user.uid, 'test_uid');
      expect(user.email, 'test_email2');
    });

    test('updatePassword', () async {
      await user.updatePassword('test_password', 'test_verifyCode', AuthProviderType.email);
      expect(mockCall.arguments, {'password':'test_password', 'verifyCode':'test_verifyCode', 'provider':12});
      expect(user.uid, 'test_uid');
      expect(user.email, 'test_email2');
    });

    test('getUserExtra', () async {
      AGCUserExtra userExtra = await user.userExtra;
      expect(user.uid, 'test_uid');
      expect(userExtra.createTime, 'test_time');
      expect(userExtra.lastSignInTime, 'test_signInTime');
    });

    test('getToken', () async {
      TokenResult tokenResult = await user.getToken();
      expect(mockCall.arguments, {'forceRefresh':false});
      expect(tokenResult.token, 'test_token');
      expect(tokenResult.expirePeriod, 1234);
    });

  });

}
